'  Stereo Tone Generator
'  Copyright 2007 Raymond Allen
'  Settings for Demo Board Audio Output:  Right Pin# = 10, Left Pin# = 11  


CON _clkmode = xtal1 + pll16x
    _xinfreq = 5_000_000       '80 MHz

    PiBy2 = 2048   'because sine table of 2049 samples is pi/2 degrees


VAR long parameter
    long table1[2000]
    long stack1[100]
    long table2[2000]
    long stack2[100]
    

PUB Main

   'Launch each tone generator in it's own cog

  'Note:  Frequency must be between 8 and 2000 Hz   

  COGNEW(Tone(400, 10, @table1, 2),@stack1)      'Right channel of demo board at 400 Hz with lots of attenuation
  COGNEW(Tone(500, 11, @table2, 2),@stack2)      'Left channel of demo board at 500 Hz with lots of attenuation


PUB TONE(Freq, Pin, pTable, attenuation)|n,i,nextCnt

  'Freq is desired freqency in Hz
  'Pin is pin to output on (See Demo board schematic for resistor and capacitor you need to get proper output)
  'pTable is array where the sine data will be stored
  'attenuation is a dividing factor for ampitude (i.e., 0,1,2,3,4 are increasing values of attenation)
  '  Use a higher attenuation if volume is too loud...  

  DIRA[Pin]~~                              'Set Pin to output
  CTRA:= %00110 << 26 + 0<<9 + Pin         'NCO/PWM Single-Ended APIN=Pin (BPIN=0 always 0) 

  'Generate list of outputs
  n:=16000/freq  '#samples per output period  (16kHz sample rate)

  if n>2000   'check to keep from overflowing table if frequency too low...
    return

  if n<1     'check at least one or two samples...
    return     
    
  repeat i from 0 to n-1
    long[pTable+i*4]:=SINE(4*PiBy2*i/n, attenuation) 'Using the ROM's SINE Table
    
  n--
  i:=0
  NextCnt:=cnt+5000
  'This loop updates the counter with the new desired output level
  repeat
    repeat i from 0 to n*4 step 4
      NextCnt+=5000   ' need this to be 5000 for 16KSPS   @ 80 MHz
      waitcnt(NextCnt)
        FRQA:=long[pTable+i]
 

PUB SINE(d, attenuation):sin|q,a,s
'This generates the sinusoidal waveform to output using ROM sine table
'd is angle between 0 and PiBy2*4

  'get quadrant
  q:=d/(PiBy2)
  'get remainder
  a:=d//(PiBy2)

  case q
    1:
       a:=PiBy2-a
       q:=0
    3:
       a:=PiBy2-a
       q:=2       

  s:=word[$E000+a<<1]>>(1+attenuation)     
  case q
    0:
       sin:= (s+$8000)<<16  '
    2:      
       sin:= ($8000-s)<<16  
  return sin 
  




